@@ -43,6 +43,7 @@ if ($@) {
 my $opt_help = 0;
 my $opt_target = "samba";
 my $opt_quick = 0;
+my $opt_quic_ko_wrapper = 0;
 my $opt_socket_wrapper = 0;
 my $opt_socket_wrapper_pcap = undef;
 my $opt_socket_wrapper_keep_pcap = undef;
@@ -55,13 +56,18 @@ my @opt_include_env = ();
 my $opt_testenv = 0;
 my $opt_list = 0;
 my $opt_mitkrb5 = 0;
+my $opt_default_ldb_backend = "mdb";
 my $opt_resetup_env = undef;
 my $opt_load_list = undef;
+my $opt_libquic_ko_wrapper_so_path = "";
 my $opt_libnss_wrapper_so_path = "";
 my $opt_libresolv_wrapper_so_path = "";
 my $opt_libsocket_wrapper_so_path = "";
 my $opt_libuid_wrapper_so_path = "";
+my $opt_libpam_wrapper_so_path = "";
+my $opt_libpam_matrix_so_path = "";
 my $opt_libasan_so_path = "";
+my $opt_libcrypt_so_path = "";
 my $opt_use_dns_faking = 0;
 my @testlists = ();
 
@@ -124,10 +130,15 @@ sub run_testsuite($$$$$)
 	my ($envname, $name, $cmd, $i, $totalsuites) = @_;
 	my $pcap_file = $target->setup_pcap($name);
 
+	Subunit::report_time();
 	Subunit::start_testsuite($name);
 	Subunit::progress_push();
 	Subunit::report_time();
-	system($cmd);
+	# Enable pipefail so that we catch failing testsuites that are part of a
+	# pipeline (typically, piped through filter-subunit). This won't catch
+	# any testsuite failures that are turned into testsuite-xfails by
+	# filter-subunit.
+	system("bash", "-o", "pipefail", "-c", $cmd);
 	Subunit::report_time();
 	Subunit::progress_pop();
 
@@ -208,7 +219,7 @@ DNS:
 
 Target Specific:
  --socket-wrapper-pcap      save traffic to pcap directories
- --socket-wrapper-keep-pcap keep all pcap files, not just those for tests that 
+ --socket-wrapper-keep-pcap keep all pcap files, not just those for tests that
                             failed
  --socket-wrapper           enable socket wrapper
 
@@ -225,6 +236,7 @@ my $result = GetOptions (
 		'help|h|?' => \$opt_help,
 		'target=s' => \$opt_target,
 		'prefix=s' => \$prefix,
+		'quic-ko-wrapper' => \$opt_quic_ko_wrapper,
 		'socket-wrapper' => \$opt_socket_wrapper,
 		'socket-wrapper-pcap' => \$opt_socket_wrapper_pcap,
 		'socket-wrapper-keep-pcap' => \$opt_socket_wrapper_keep_pcap,
@@ -239,15 +251,20 @@ my $result = GetOptions (
 		'testenv' => \$opt_testenv,
 		'list' => \$opt_list,
 		'mitkrb5' => \$opt_mitkrb5,
+		'default-ldb-backend=s' => \$opt_default_ldb_backend,
 		'resetup-environment' => \$opt_resetup_env,
 		'testlist=s' => \@testlists,
 		'random-order' => \$opt_random_order,
 		'load-list=s' => \$opt_load_list,
+		'quic_ko_wrapper_so_path=s' => \$opt_libquic_ko_wrapper_so_path,
 		'nss_wrapper_so_path=s' => \$opt_libnss_wrapper_so_path,
 		'resolv_wrapper_so_path=s' => \$opt_libresolv_wrapper_so_path,
 		'socket_wrapper_so_path=s' => \$opt_libsocket_wrapper_so_path,
 		'uid_wrapper_so_path=s' => \$opt_libuid_wrapper_so_path,
+		'pam_wrapper_so_path=s' => \$opt_libpam_wrapper_so_path,
+		'pam_matrix_so_path=s' => \$opt_libpam_matrix_so_path,
 		'asan_so_path=s' => \$opt_libasan_so_path,
+		'crypt_so_path=s' => \$opt_libcrypt_so_path,
 		'use-dns-faking' => \$opt_use_dns_faking
 	    );
 
@@ -277,6 +294,7 @@ $ENV{SAMBA_DEPRECATED_SUPPRESS} = 1;
 # see also bootstrap/config.py
 $ENV{TZ} = "UTC";
 $ENV{LC_ALL} = $ENV{LANG} = "en_US.utf8";
+$ENV{LANGUAGE} = "en_US";
 
 my $bindir_abs = abs_path($bindir);
 
@@ -285,6 +303,7 @@ my $torture_maxtime = ($ENV{TORTURE_MAXTIME} or 1200);
 $prefix =~ s+//+/+;
 $prefix =~ s+/\./+/+;
 $prefix =~ s+/$++;
+$prefix = abs_path($prefix);
 
 die("using an empty prefix isn't allowed") unless $prefix ne "";
 
@@ -300,19 +319,17 @@ chmod 0700, $prefix;
 # We need to have no umask limitations for the tests.
 umask 0000;
 
-my $prefix_abs = abs_path($prefix);
 my $tmpdir_abs = abs_path("$prefix/tmp");
 mkdir($tmpdir_abs, 0777) unless -d $tmpdir_abs;
 
 my $srcdir_abs = abs_path($srcdir);
 
-die("using an empty absolute prefix isn't allowed") unless $prefix_abs ne "";
-die("using '/' as absolute prefix isn't allowed") unless $prefix_abs ne "/";
+die("using an empty absolute prefix isn't allowed") unless $prefix ne "";
+die("using '/' as absolute prefix isn't allowed") unless $prefix ne "/";
 
 $ENV{SAMBA_SELFTEST} = "1";
 
 $ENV{PREFIX} = $prefix;
-$ENV{PREFIX_ABS} = $prefix_abs;
 $ENV{SRCDIR} = $srcdir;
 $ENV{SRCDIR_ABS} = $srcdir_abs;
 $ENV{BINDIR} = $bindir_abs;
@@ -346,9 +363,25 @@ my $ld_preload = $ENV{LD_PRELOAD};
 
 if ($opt_libasan_so_path) {
 	if ($ld_preload) {
-		$ld_preload = "$opt_libasan_so_path:$ld_preload";
+		if ($opt_libcrypt_so_path) {
+			$ld_preload = "$opt_libasan_so_path:$opt_libcrypt_so_path:$ld_preload";
+		} else {
+			$ld_preload = "$opt_libasan_so_path:$ld_preload";
+		}
 	} else {
-		$ld_preload = "$opt_libasan_so_path";
+		if ($opt_libcrypt_so_path) {
+			$ld_preload = "$opt_libasan_so_path:$opt_libcrypt_so_path";
+		} else {
+			$ld_preload = "$opt_libasan_so_path";
+		}
+	}
+}
+
+if ($opt_libquic_ko_wrapper_so_path) {
+	if ($ld_preload) {
+		$ld_preload = "$ld_preload:$opt_libquic_ko_wrapper_so_path";
+	} else {
+		$ld_preload = "$opt_libquic_ko_wrapper_so_path";
 	}
 }
 
@@ -384,6 +417,14 @@ if ($opt_libuid_wrapper_so_path) {
 	}
 }
 
+if ($opt_libpam_wrapper_so_path) {
+	if ($ld_preload) {
+		$ld_preload = "$ld_preload:$opt_libpam_wrapper_so_path";
+	} else {
+		$ld_preload = "$opt_libpam_wrapper_so_path";
+	}
+}
+
 if (defined($ENV{USE_NAMESPACES})) {
 	print "Using linux containerization for selftest testenv(s)...\n";
 
@@ -403,15 +444,13 @@ $ENV{UID_WRAPPER} = 1;
 # We are already hitting the limit, so double it.
 $ENV{NSS_WRAPPER_MAX_HOSTENTS} = 200;
 
-# Disable RTLD_DEEPBIND hack for Samba bind dlz module
-#
-# This is needed in order to allow the ldb_*ldap module
-# to work with a preloaded socket wrapper.
-$ENV{LDB_MODULES_DISABLE_DEEPBIND} = 1;
+if ($opt_quic_ko_wrapper) {
+	$ENV{QUIC_KO_WRAPPER} = 1;
+}
 
 my $socket_wrapper_dir;
 if ($opt_socket_wrapper) {
-	$socket_wrapper_dir = SocketWrapper::setup_dir("$prefix_abs/w", $opt_socket_wrapper_pcap);
+	$socket_wrapper_dir = SocketWrapper::setup_dir("$prefix/w", $opt_socket_wrapper_pcap);
 	print "SOCKET_WRAPPER_DIR=$socket_wrapper_dir\n";
 } elsif (not $opt_list) {
 	 unless ($< == 0) {
@@ -431,6 +470,10 @@ if ($opt_mitkrb5 == 1) {
 	$ENV{KRB5RCACHETYPE} = "none";
 }
 
+# Enable support for SHA1 in OpenSSL
+# This is required e.g. for pkinit sha1 tests
+$ENV{OPENSSL_ENABLE_SHA1_SIGNATURES} = 1;
+
 # After this many seconds, the server will self-terminate.  All tests
 # must terminate in this time, and testenv will only stay alive this
 # long
@@ -450,7 +493,9 @@ if (defined($ENV{SMBD_MAXTIME}) and $ENV{SMBD_MAXTIME} ne "") {
 
 $target = new Samba($bindir, $srcdir, $server_maxtime,
 		    $opt_socket_wrapper_pcap,
-		    $opt_socket_wrapper_keep_pcap);
+		    $opt_socket_wrapper_keep_pcap,
+		    $opt_libpam_matrix_so_path,
+		    $opt_default_ldb_backend);
 unless ($opt_list) {
 	if ($opt_target eq "samba") {
 		$testenv_default = "ad_dc";
@@ -464,14 +509,14 @@ sub read_test_regexes($)
 	my ($name) = @_;
 	my @ret = ();
 	open(LF, "<$name") or die("unable to read $name: $!");
-	while (<LF>) { 
-		chomp; 
+	while (<LF>) {
+		chomp;
 		next if (/^#/);
 		if (/^(.*?)([ \t]+)\#([\t ]*)(.*?)$/) {
 			push (@ret, [$1, $4]);
 		} else {
 			s/^(.*?)([ \t]+)\#([\t ]*)(.*?)$//;
-			push (@ret, [$_, undef]); 
+			push (@ret, [$_, undef]);
 		}
 	}
 	close(LF);
@@ -492,7 +537,7 @@ foreach (@opt_include) {
 $ENV{SOCKET_WRAPPER_IPV4_NETWORK} = "10.53.57.0";
 my $interfaces = Samba::get_interfaces_config("client", 6);
 
-my $clientdir = "$prefix_abs/client";
+my $clientdir = "$prefix/client";
 
 my $conffile = "$clientdir/client.conf";
 $ENV{SMB_CONF_PATH} = $conffile;
@@ -535,7 +580,7 @@ sub write_clientconf($$$)
 	# each user has a USER-${USER_PRINCIPAL_NAME}-cert.pem and
 	# USER-${USER_PRINCIPAL_NAME}-private-key.pem symlink
 	# We make a copy here and make the certificated easily
-	# accessable in the client environment.
+	# accessible in the client environment.
 	my $mask = umask;
 	umask 0077;
 	opendir USERS, "${ca_users_dir}" or die "Could not open dir '${ca_users_dir}': $!";
@@ -650,7 +695,7 @@ if ($#testlists == -1) {
 	die("No testlists specified");
 }
 
-$ENV{SELFTEST_PREFIX} = "$prefix_abs";
+$ENV{SELFTEST_PREFIX} = "$prefix";
 $ENV{SELFTEST_TMPDIR} = "$tmpdir_abs";
 $ENV{TMPDIR} = "$tmpdir_abs";
 $ENV{TEST_DATA_PREFIX} = "$tmpdir_abs";
@@ -882,8 +927,8 @@ sub teardown_env($)
 }
 
 # This 'global' file needs to be empty when we start
-unlink("$prefix_abs/dns_host_file");
-unlink("$prefix_abs/hosts");
+unlink("$prefix/dns_host_file");
+unlink("$prefix/hosts");
 
 if ($opt_random_order) {
 	require List::Util;
@@ -981,7 +1026,7 @@ $envvarstr
 			next;
 		}
 
-		# Generate a file with the individual tests to run, if the 
+		# Generate a file with the individual tests to run, if the
 		# test runner for this test suite supports it.
 		if ($individual_tests and $individual_tests->{$name}) {
 			if ($$_[3]) {
