@@ -115,7 +115,7 @@ static int write_block( REGF_FILE *file, prs_struct *ps, uint32_t offset )
 
 static int read_block( REGF_FILE *file, prs_struct *ps, uint32_t file_offset, uint32_t block_size )
 {
-	int bytes_read, returned;
+	ssize_t bytes_read, returned;
 	char *buffer;
 	SMB_STRUCT_STAT sbuf;
 
@@ -182,6 +182,10 @@ static int read_block( REGF_FILE *file, prs_struct *ps, uint32_t file_offset, ui
 			return False;
 		}
 
+		if (returned < 0 || bytes_read > SSIZE_MAX - returned) {
+			DBG_ERR("Integer overflow\n");
+			return false;
+		}
 		bytes_read += returned;
 	}
 
@@ -1170,7 +1174,7 @@ static bool next_nk_record( REGF_FILE *file, REGF_HBIN *hbin, REGF_NK_REC *nk, b
 }
 
 /*******************************************************************
- Intialize the newly created REGF_BLOCK in *file and write the
+ Initialize the newly created REGF_BLOCK in *file and write the
  block header to disk
 *******************************************************************/
 
@@ -1190,7 +1194,7 @@ static bool init_regf_block( REGF_FILE *file )
 
 	unix_to_nt_time( &file->mtime, time(NULL) );
 
-	/* hard coded values...no diea what these are ... maybe in time */
+	/* hard coded values...no idea what these are ... maybe in time */
 
 	file->unknown1 = 0x2;
 	file->unknown2 = 0x1;
@@ -1236,11 +1240,10 @@ out:
 {
 	REGF_FILE *rb;
 
-	if ( !(rb = SMB_MALLOC_P(REGF_FILE)) ) {
+	if (!(rb = SMB_CALLOC_ARRAY(REGF_FILE, 1))) {
 		DEBUG(0,("ERROR allocating memory\n"));
 		return NULL;
 	}
-	ZERO_STRUCTP( rb );
 	rb->fd = -1;
 	rb->ignore_checksums = false;
 
@@ -1427,7 +1430,7 @@ REGF_NK_REC* regfio_rootkey( REGF_FILE *file )
 }
 
 /*******************************************************************
- This acts as an interator over the subkeys defined for a given
+ This acts as an iterator over the subkeys defined for a given
  NK record.  Remember that offsets are from the *first* HBIN block.
 *******************************************************************/
 
